/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/16 12:10:02 $
 // $Revision: 1.7 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/

#include "backinlay.h"
using namespace std;

/**
 * Constructors/Destructors
 */

/**
 * Constructor, protected, only to be called from derived subclasses to save coding action
 */
BackInlay::BackInlay (PSDoc* newsheet, FontSet* newfonts, CDInfo& newcontent) {
	fonts = newfonts;
	sheet = newsheet;
	content = newcontent;
	ref_X=15.0;
	ref_Y=15.0;
	PS_setfont(sheet,fonts->titlefont.psfont_handle,fonts->titlefont.size);
	layout();
}


/**
 * Methods
 */
/**
 * Get the value of spinewidth
 * width of spine 6mm
 * @return the value of spinewidth
 */
float BackInlay::get_spinewidth ( ) {
	return spinewidth;
}

/**
 * Get the value of width
 * Width of inlay 150mm
 * @return the value of width
 */
float BackInlay::get_width ( ) {
	return width;
}

/**
 * Get the value of height
 * height of inlay 120 mm
 * @return the value of height
 */
float BackInlay::get_height ( ) {
	return height;
}

/**
 * Get the value of fonts
 * pointer to the beginning on the fontarray used on this document
 * @return the value of fonts
 */
FontSet* BackInlay::get_fonts ( ) {
	return fonts;
}
/**
 * Set the value of fonts
 * pointer to the beginning on the fontarray used on this document
 * @param value the value of fonts
 */
void BackInlay::set_fonts (FontSet* value ) {
	fonts = value;
}
/**
 * Get the value of ref_X
 * X coordinate to start sheet (upper left)
 * @return the value of ref_X
 */
float BackInlay::get_ref_X ( ) {
	return ref_X;
}
/**
 * Set the value of ref_X
 * X coordinate to start sheet (upper left)
 * @param value the value of ref_X
 */
void BackInlay::set_ref_X (float value ) {
	ref_X = value;
}
/**
 * Get the value of ref_Y
 * Y coordinate to start sheet (upper left)
 * @return the value of ref_Y
 */
float BackInlay::get_ref_Y ( ) {
	return ref_Y;
}
/**
 * Set the value of ref_Y
 * Y coordinate to start sheet (upper left)
 * @param value the value of ref_Y
 */
void BackInlay::set_ref_Y (float value ) {
	ref_Y = value;
}
/**
 * Get the value of sheet
 * pdfhandle we write on
 * @return the value of sheet
 */
PSDoc* BackInlay::get_sheet ( ) {
	return sheet;
}
/**
 * Set the value of sheet
 * pdfhandle we write on
 * @param value the value of sheet
 */
void BackInlay::set_sheet (PSDoc* value ) {
	sheet = value;
}


/**
 * write all stuff like composer and date
 */
void  BackInlay::writeGenericInfo(){
	time_t rawtime;
	struct tm * timeinfo;
	time ( &rawtime );
	timeinfo = localtime ( &rawtime );
	string str = content.get_composer();
	str += " - ";
	str += asctime(timeinfo);
	PS_setcolor(sheet, "stroke", "rgb", 0.3, 0.3, 0.3, 0.0);
	PS_setcolor(sheet, "fill", "rgb", 0.5, 0.5, 0.5, 0.0);
	PS_setfont(sheet, fonts->genericfont.psfont_handle,fonts->genericfont.size);
	PS_show_boxed(sheet,str.c_str(),spinewidth+2,0.5,width-(2*spinewidth+4),GENERICFONTSIZE,"right",NULL);
}

/**
 * Write artist & album title using fontface titlefont
 */
void BackInlay::writeTitle () {
	// temporary spinestring
	string str = content.get_artist();
	str +=" - ";
	str += content.get_title();
	PS_setfont(sheet,fonts->titlefont.psfont_handle,fonts->titlefont.size);
	PS_show_boxed(sheet,str.c_str(),spinewidth+1,109,width-(2*spinewidth+2),7.5,"center",NULL);
}
/**
 * Write subtitle if applicable under "artist & album" title using fontface #1(subtitlefont)
 */
void BackInlay::writeSubtitle () {
	PS_setfont(sheet, fonts->subtitlefont.psfont_handle,fonts->subtitlefont.size);
	PS_show_boxed(sheet,content.get_subtitle().c_str(),spinewidth+2,102,width-(2*spinewidth+4),6.5,"center",NULL);
}
/**
 * Write artist & album title on both spines using fontface #2(spinefont)
 */
void BackInlay::writeSpine () {
	// temporary spinestring
	string str = content.get_artist();
	str +=" - ";
	str += content.get_title();
	//left spine
	PS_rotate(sheet,90.0);
	PS_setfont(sheet,fonts->spinefont.psfont_handle,fonts->spinefont.size);
	PS_show_boxed(sheet,str.c_str(),5,-spinewidth-1,height-10,spinewidth,"center",NULL);
	// right spine
	PS_rotate(sheet,180.0);
	PS_show_boxed(sheet,str.c_str(),-height+5,width-spinewidth-1, height-10,spinewidth,"center",NULL);
	//reset the coordinates
	PS_rotate(sheet,90.0);
}
/**
 * draw a box so  the user can cut it easily with scissors
 */
void BackInlay::drawOutline () {
	PS_setcolor(sheet, "stroke", "rgb", 0.0, 0.0, 0.0, 0.0);
	PS_setcolor(sheet, "fill", "rgb", 1.0, 1.0, 1.0, 0.0);
	PS_setlinewidth(sheet, 0.1);
	PS_rect(sheet,0.0,0.0,width,height);
	PS_moveto(sheet,spinewidth,0.0);
	PS_lineto(sheet,spinewidth,height);
	PS_moveto(sheet,width-spinewidth,0.0);
	PS_lineto(sheet,width-spinewidth,height);
	PS_fill_stroke(sheet);
}
/**
 * write tracklist 
 * abstract here because we don't know how we want ot write it 
 */
void BackInlay::writeTracklist () {
	// Build the information text
	string str,timebuf,timestr, numbuf, numstr;
	vector<TrackInfo> vex = content.get_tracks();
	vector<TrackInfo>::iterator it = vex.begin();
	int i =1;
	while(it != vex.end()){
		itoa(numbuf,i);			// Convert a integer to a string this is an inline function defined in cdinlay_util
		numstr += numbuf+".\n";
		str += "["+(*it).get_artist()+"] "+(*it).get_title()+"  \n";
		timeformat(timebuf,(*it).get_time());
		timestr += timebuf +"\n";
		PS_set_value(sheet,"textx",9.0);
		PS_set_value(sheet,"leading",fonts->titlefont.size);
		it++;
		i++;
	}
	PS_set_parameter(sheet, "linebreak", "true");
	PS_setfont(sheet,fonts->trackfont.psfont_handle,fonts->trackfont.size);
	PS_show_boxed(sheet,numstr.c_str(),spinewidth+4,3,5.5,99,"right",NULL);
	PS_show_boxed(sheet,str.c_str(),spinewidth+10,3,width-(2*spinewidth+15),99,"left",NULL);
	PS_show_boxed(sheet,timestr.c_str(),width-(2*spinewidth+15),3,15,99,"right",NULL);
	PS_set_parameter(sheet, "linebreak", "false");
}
/**
 * method that is called to put all the features together and place it on the PS within a box for cutting
 */
void BackInlay::layout () {
	drawOutline();
	writeSpine();
	writeTitle();
	writeSubtitle();
	writeTracklist();
	writeGenericInfo();
	
}

