/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/16 12:10:02 $
 // $Revision: 1.7 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/


#ifndef BACKINLAY_H
#define BACKINLAY_H
#include <string>
#include <time.h>
#include <iostream>


#include <cstdlib>
extern "C"{	
	#include <libps/pslib.h>		// used for generation of Postscript documents
}
#include "cdinfo.h"
#include "trackinfo.h"
#include "cdinlay_types.h"
#include "cdinlay_util.h"
/**
 * Class BackInlay
 * Class to hold it all together and coordinate the pagelayout write the basic structure that is always the same such as spine and title
 */
class BackInlay {
	/**
	 * Public stuff
	 */
 public:
	/**
	 * Constructors
	 */
	/**
	 * Constructor
	 */
	BackInlay (PSDoc* newsheet, FontSet* newfonts, CDInfo& newcontent);
		
	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of content
	 * All information that will be displayed on the CD cover
	 * @return the value of content
	 */
	CDInfo get_content ( );
		
	
	/**
	 * Set the value of content
	 * All information that will be displayed on the CD cover
	 * @param value the value of content
	 */
	void set_content (CDInfo& value );
		

	
	/**
	 * Get the value of spinewidth
	 * width of spine 6mm
	 * @return the value of spinewidth
	 */
	float get_spinewidth ( );
		
	/**
	 * Get the value of width
	 * Width of inlay 150mm
	 * @return the value of width
	 */
	float get_width ( );
		
	/**
	 * Get the value of height
	 * height of inlay 120 mm
	 * @return the value of height
	 */
	float get_height ( );
		

	/**
	 * Get the value of fonts
	 * pofloater to the beginning on the fontarray used on this document
	 * @return the value of fonts
	 */
	FontSet* get_fonts ( );
		
	
	/**
	 * Set the value of fonts
	 * pofloater to the beginning on the fontarray used on this document
	 * @param value the value of fonts
	 */
	void set_fonts (FontSet* value );
		
	
	/**
	 * Get the value of ref_X
	 * X coordinate to start sheet (upper left)
	 * @return the value of ref_X
	 */
	float get_ref_X ( );
		
	
	/**
	 * Set the value of ref_X
	 * X coordinate to start sheet (upper left)
	 * @param value the value of ref_X
	 */
	void set_ref_X (float value );
		
	
	/**
	 * Get the value of ref_Y
	 * Y coordinate to start sheet (upper left)
	 * @return the value of ref_Y
	 */
	float get_ref_Y ( );
		
	
	/**
	 * Set the value of ref_Y
	 * Y coordinate to start sheet (upper left)
	 * @param value the value of ref_Y
	 */
	void set_ref_Y (float value );
		
	
	/**
	 * Get the value of sheet
	 * pdfhandle we write on
	 * @return the value of sheet
	 */
	PSDoc* get_sheet ( );
		
	
	/**
	 * Set the value of sheet
	 * pdfhandle we write on
	 * @param value the value of sheet
	 */
	void set_sheet (PSDoc* value );

	
	/**
	 * method that is called to put all the features together and place it on the PS within a box for cutting
	 */
	void  layout ();

	/**
	 * Private stuff
	 */
 private:
	/**
	 * Fields
	 */
	/**
  * All information that will be displayed on the CD cover
	 */
	 CDInfo content;
	 
	/**
  	* width of spine (6mmi)
	 */
	 static const float spinewidth=6.5;
	/**
	  * Width of inlay (150mm)
	 */
	static const float width=151.5;
	/**
	 * height of inlay (120 mm)
	 */
	static const float height=119.0;
	/**
	 * pofloater to the beginning on the fontarray used on this document
	 */
	FontSet* fonts;
	/**
	 * X coordinate to start sheet (upper left)
	 */
	float ref_X;
	/**
	 * Y coordinate to start sheet (upper left)
	 */
	float ref_Y;
	/**
	 * pdfhandle we write on
	 */
	PSDoc* sheet;
	
	/**
	 * Operations
	 */
		
	/**
	 * Operations
	 */
	/**
	 * Write artist & album title using fontface #0(titlefont)
	 */
	void  writeTitle ();
		
	
	/**
	 * Write subtitle if applicable under "artist & album" title using fontface #1(subtitlefont)
	 */
	void  writeSubtitle ();
		
	
	/**
	 * Write artist & album title on both spines using fontface #2(spinefont)
	 */
	void  writeSpine ();
		
	
	/**
	 * draw a box so  the user can cut it easily with scissors
	 */
	void  drawOutline ();

	/**
	 * write all stuff like composer and date
	 */
	void  writeGenericInfo();
		
	
	/**
	 * write tracklist
	 * abstract here because we don't know how we want ot write it
	 */
	void  writeTracklist ();
		
	
};
#endif //BACKINLAY_H

