/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/14 19:15:19 $
 // $Revision: 1.4 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/

#ifndef CDINFO_H
#define CDINFO_H
#include <string>
#include <vector>
#include <iostream>
#include "trackinfo.h"


using namespace std;
/**
 * Class CDInfo
 * Container for all important infos of a CD, holding the tracklist
 */
class CDInfo {
	/**
	 * Public stuff
	 */
 public:
	/**
	 * Constructors
	 */
	/**
	 * Empty Constructor
	 */
	CDInfo ( ) { }

	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of title
	 * name of this CD
	 * @return the value of title
	 */
	string get_title ( );
		
	
	/**
	 * Set the value of title
	 * name of this CD
	 * @param value the value of title
	 */
	void set_title (string value );
		
	
	/**
	 * Get the value of subtitle
	 * additional information for the title will be shown in a smaller font
	 * @return the value of subtitle
	 */
	string get_subtitle ( );
		
	
	/**
	 * Set the value of subtitle
	 * additional information for the title will be shown in a smaller font
	 * @param value the value of subtitle
	 */
	void set_subtitle (string value );
		
	
	/**
	 * Get the value of artist
	 * Original singer/band that published this album - leave it fro sampler
	 * @return the value of artist
	 */
	string get_artist ( );
		
	
	/**
	 * Set the value of artist
	 * Original singer/band that published this album - leave it fro sampler
	 * @param value the value of artist
	 */
	void set_artist (string value );
		
	
	/**
	 * Get the value of setnumber
	 * Edition, volume (if this is a multi-CD Compilation)
	 * @return the value of setnumber
	 */
	int get_setnumber ( );
		
	
	/**
	 * Set the value of setnumber
	 * Edition, volume (if this is a multi-CD Compilation)
	 * @param value the value of setnumber
	 */
	void set_setnumber (int value );
		
	
	/**
	 * Get the value of tracks
	 * container for all tracks
	 * @return the value of tracks
	 */
	vector<TrackInfo> get_tracks ( );
		
	
	/**
	 * Set the value of tracks
	 * container for all tracks
	 * @param value the value of tracks
	 */
	void set_tracks (vector<TrackInfo> value );
		
	
	/**
	 * Get the value of composer
	 * Composer / publisher  of the CD (mostly who ripped this thing)
	 * @return the value of composer
	 */
	string get_composer ( );
		
	
	/**
	 * Set the value of composer
	 * Composer / publisher  of the CD (mostly who ripped this thing)
	 * @param value the value of composer
	 */
	void set_composer (string value );
		
	
	/**
	 * Get the value of publishdate
	 * Date CD was published
	 * @return the value of publishdate
	 */
	string get_publishdate ( );
		
	
	/**
	 * Set the value of publishdate
	 * Date CD was published
	 * @param value the value of publishdate
	 */
	void set_publishdate (string value );
		

	
	/**
	 * Operations
	 */
	/**
	 * appends the new track to tracklist
	 */
	void  addTrack (TrackInfo newTrack);
		
	
	/**
	 * add the new track to tracklist at position index throws exception if no such position
	 */
	void  addTrackAt (int index, TrackInfo &newTrack);
		
	
	/**
	 * delete a track from tracklist
	 */
	void  deleteTrack (int index);
		
	
	/**
	 * delete Track when we don't know what number but what object (this specific track)
	 */
	void  deleteTrack (TrackInfo &track);

 private:
	/**
	 * Fields
	 */
	/**
	 * name of this CD
	 */
	string title;
	/**
	 * additional information for the title will be shown in a smaller font
	 */
	string subtitle;
	/**
	 * Original singer/band that published this album - leave it fro sampler
	 */
	string artist;
	/**
	 * Edition, volume (if this is a multi-CD Compilation)
	 */
	int setnumber;
	/**
	 * container for all tracks
	 */
	vector<TrackInfo> tracks;
	/**
	 * Composer / publisher  of the CD (mostly who ripped this thing)
	 */
	string composer;
	/**
	 * Date CD was published
	 */
	string publishdate;

	
	/**
	 * Operations
	 */
};
#endif //CDINFO_H

