/******************************************************************************
// Synopsis:
//-----------------------
// Dependencies:
//
//-----------------------
// CVS:
// $Author: ruschi $
// $Date: 2004/11/21 17:01:37 $
// $Revision: 1.6 $ 
//-----------------------------------------------------------------------------
//  Copyright (C) 2004 by Thomas Ruschival                               
//  ruschi@doomgarage.de                                                 
//                                                                       
//  This program is free software; you can redistribute it and/or modify 
//  it under the terms of the GNU General Public License as published by 
//  the Free Software Foundation; either version 2 of the License, or    
//  (at your option) any later version.                                  
//                                                                       
//  This program is distributed in the hope that it will be useful,      
//  but WITHOUT ANY WARRANTY; without even the implied warranty of       
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
//  GNU General Public License for more details.                         
//                                                                       
//  You should have received a copy of the GNU General Public License    
//  along with this program; if not, write to the                        
//  Free Software Foundation, Inc.,                                      
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
*******************************************************************************/

#include "cdinfofactory.h"
using namespace std;

/**
 * Constructors/Destructors
 */

/**
 * Constructor with Container for Infos
 * @param cdcontent Container for Trackinfo & CDinformation
 */
CDInfoFactory::CDInfoFactory (CDInfo& cdcontent,string infile) {
	content = cdcontent;
	source = infile;
	loadReaders(); // loading dynamic plugins
	cur_reader = getReader(infile);
	cur_reader->readCDInfo(content);
}

   
/**
 * Methods
 */
 
/**
 * Get the value of content
 * Container where the readers put their information
 * @return the value of content
 */
CDInfo CDInfoFactory::get_content ( ) {
	return content;
}

/**
 * Set the value of content
 * Container where the readers put their information
 * @param value the value of content
 */
void CDInfoFactory::set_content (CDInfo& value ) {
	content=value;
}
/**
 * Get the value of source
 * Location to to read it's not opened as a istram or stored as file because readers have to decide how to open and handle the Information
 * @return the value of  source
 */
string CDInfoFactory::get_source( ) {
	return source;
}
/**
 * Set the value of source
 * Location to to read it's not opened as a istram or stored as file because readers have to decide how to open and handle the Information
 * @param value the value of  source
 */
void CDInfoFactory::set_source (string value ) {
	source = value;
}


/**
 * return a fitting reader for the specified infile
 * getting Information from extension
 * @param infile source to read
 * @return cur_reader the suitable reader for this problem. or 0 if no reader is found
*/
CDInfoReader* CDInfoFactory::getReader (string infile) {
	source = infile;
	// flag if any reader suits this extension ->> replace it later with exception
	int found =0;
	vector<CDInfoReader*>::iterator it = readers.begin();
	while( it != readers.end() ){
		if((*it)->matches(infile)){
			found = 1;
			cur_reader = *it;
			break;
		}		
		it++;
 	}
	if(found == 0){
		cout << "! ERROR: No Reader found for filetype: "<< infile  << endl;
		exit(1);
	}
	cout << "--> Using reader:" <<  cur_reader->get_name() << endl;
	return cur_reader;
}

/**
 * content will be newly read from the specified source
 */

CDInfo CDInfoFactory::get_content (string infile) {
	cur_reader = getReader(infile);
	cur_reader->readCDInfo(content);
	return content;
}

/**
 * dynamically load specific CDInfoReader classes into the
 * reader array.
 */
void CDInfoFactory::loadReaders () {
	DIR *dir;				// handle for directory
	struct dirent *entry;	// file we just read from directory

	dir = opendir(PLUGIN_PATH);
	if (dir == NULL) {
		cout << "Error: opendir()"<<  PLUGIN_PATH << endl;
	}
	
	// Browse the directory
	while ((entry = readdir(dir)) != NULL) {
		if (entry->d_type == DT_REG) {
			// match the filename for .so ending
			if (!fnmatch(FILENAME_FILTER, entry->d_name, 0)) {
				/**
				 * do the macigal dynamic loading of the parsers
				 */
				// build the filename
				string filename(PLUGIN_PATH);
				filename+=entry->d_name;
				void* tmpreader = dlopen(filename.c_str(), RTLD_LAZY);
				if (!tmpreader) {
					cerr << "Cannot load library: " << filename << " : " << dlerror() << '\n';
				}

				// load the symbols (I don't really know what happens here)
				create_t* create_plugin = (create_t*) dlsym(tmpreader, "create");
				destroy_t* destroy_plugin = (destroy_t*) dlsym(tmpreader, "destroy");
				if (!create_plugin || !destroy_plugin) {
					cerr << "Cannot load symbols: " << filename << " : " << dlerror() << '\n';
				}
				// create an instance of the class
				cur_reader = create_plugin(source); //call the constructor wrapper
				readers.push_back(cur_reader); // store our reader in the vector
			}
		}	
	}
	closedir(dir);
}

