/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/21 17:01:37 $
 // $Revision: 1.4 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/

#ifndef CDINFOREADER_H
#define CDINFOREADER_H

#include <string>
#include "trackinfo.h"
#include "cdinfo.h"

using namespace std;

/**
 * Class CDInfoReader
 * Abstract class to provide several subclasses to create
 * a CDInfo Object 
 */
class CDInfoReader {
 public:
	 CDInfoReader(){};
	 
	 CDInfoReader(string infile);
	/**
	 * Accessor Methods
	 */

	/**
	 * Get the value of content
	 * container to store read data
	 * @return the value of content
	 */
	CDInfo get_content ( );
		
	/**
	 * Set the value of content
	 * container to store read data
	 * @param value the value of content
	 */
	void set_content (CDInfo& value );
		
	
	/**
	 * Get the value of name
	 * Name of the Parser PlugIn
	 * @return the value of name
	 */
	string get_name ( );

	/**
	 * Get the value of filetype
	 * filetype for which this Parserplugin is in charge of
	 * @return the value of filetype
	 */
	 string get_filetype ( );
		

	
	/**
	 * Get the value of description
	 * general information about hot this specific CDInforeader
	 * is used and other interesting stuff
	 * @return the value of description
	 */
	string get_description ( );

	
	/**
	 * Get the value of source
	 * Source of the trackinfornmation to read
	 * specific soiurce will be defined in subclasses
	 * @return the value of source
	 */
	string get_source ( );
		
	
	/**
	 * Set the value of source
	 * Source of the trackinfornmation to read
	 * specific soiurce will be defined in subclasses
	 * @param value the value of source
	 */
	void set_source (string value );
		
	/**
	 * Operations
	 */
	/**
	 * abstract method to get trackinformation from an arbitrary
	 * source
	 */
	virtual void readCDInfo (CDInfo &content) = 0;

	/**
	 * abstract method to find out if the reader is suitable for the 
	 * source
	 * @param infile string filename
	 * @return fits int 0 if false, 1 if it fits
	 */
	virtual int matches(string infile) = 0;
	
 	/**
	 * Fields
	 */
	/**
	 * container to store read data
	 */
	 CDInfo content;
	/**
	 * Name of the Parser PlugIn
	 */
	string name;
	/**
	 * filetype for which this Parserplugin is in charge of
	 */
	string filetype;
	/**
	 * general information about hot this specific CDInforeader 
	 * is used and other interesting stuff
	 */
	string description;
	/**
	 * Source of the trackinfornmation to read
	 * specific soiurce will be defined in subclasses
	 */
	 string source;
};

	
/**
 * Create a Instance of a CDInforeader, new cannot be used
 * with dynamically loaded plugins
 */

typedef CDInfoReader* create_t(string infile);
/**
 * Wrapper for the delete operation, we cannot delete an
 * object that has beed dynamiccaly created with the delete
 * operation form the outside, so we let the object destroy
 * itself
 */

typedef void destroy_t(CDInfoReader* obj);


#endif //CDINFOREADER_H

