/******************************************************************************
 // Synopsis:
 //-----------------------
 // Dependencies:
 //
 //-----------------------
 // CVS:
 // $Author: ruschi $
 // $Date: 2004/11/21 17:01:37 $
 // $Revision: 1.8 $ 
 //-----------------------------------------------------------------------------
 //  Copyright (C) 2004 by Thomas Ruschival                               
 //  ruschi@doomgarage.de                                                 
 //                                                                       
 //  This program is free software; you can redistribute it and/or modify 
 //  it under the terms of the GNU General Public License as published by 
 //  the Free Software Foundation; either version 2 of the License, or    
 //  (at your option) any later version.                                  
 //                                                                       
 //  This program is distributed in the hope that it will be useful,      
 //  but WITHOUT ANY WARRANTY; without even the implied warranty of       
 //  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 //  GNU General Public License for more details.                         
 //                                                                       
 //  You should have received a copy of the GNU General Public License    
 //  along with this program; if not, write to the                        
 //  Free Software Foundation, Inc.,                                      
 //  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
 *******************************************************************************/


#include "cdinforeader_csv.h"

using namespace std;


CDInfoReader_csv::CDInfoReader_csv(string infile):CDInfoReader(infile){
	delimiter = ";";
	name = "CSV Parser";
	filetype = "csv";
	description="Parser for colon separated Trackinfos\n";
	description+="Artist ; Album ; Title ; Genre ; Date ; Time ; Comment \n";
	description+="Version: $Revision: 1.8 $ Copyright (c) 2004 Thomas Ruschival \n";
}


CDInfoReader_csv::CDInfoReader_csv():CDInfoReader(){
	delimiter = ";";
	name = "CSV Parser";
	filetype = "csv";
	description="Parser for colon separated Trackinfos\n";
	description+="fields in this order:\n";
	description+="Artist ; Album ; Title ; Genre ; Date ; Time ; Comment \n";
	description+="Version: $Revision: 1.8 $ Copyright (c) 2004 Thomas Ruschival \n";
}


/**
 * find out if the reader is suitable for the
 * source
 * @param infile string filename
 * @return fits int 0 if false, 1 if it fits
 */
int  CDInfoReader_csv::matches(string infile){
	// find and extract the extension of the file
	int pos = infile.find_last_of('.',infile.size());
	string extension = infile.substr(pos+1,infile.size());
	// Make the fileextension lowercase
	std::transform(extension.begin(),extension.end(),extension.begin(),(int(*)(int)) tolower);
	//Get information about file; determine by extension
	if(extension == filetype){
		return 1;
	}
	else{
		return 0;
	}	
}
/**
 * Get the value of delimiter
 * character that separates the fields in the file
 * @return the value of delimiter
 */
string CDInfoReader_csv::get_delimiter ( ) {
	return delimiter;
}
/**
 * Set the value of delimiter
 * character that separates the fields in the file
 * @param value the value of delimiter
 */
void CDInfoReader_csv::set_delimiter (string value ) {
	delimiter = value;
}
/**
 * parse the file, split the lines, generate trackinfos
 */
void CDInfoReader_csv::readCDInfo (CDInfo &content) {
	int number=1;
	char buffer[256];
	ifstream examplefile(this->source.c_str());
	string line;
	vector<string> info; // container for line tokens
	if (! examplefile.is_open()){ 
		cout << "Error opening file"; exit (1); 
	}
  
	while (! examplefile.eof() )  {
		examplefile.getline (buffer,255);
		const string str(buffer);
		if(str.size() >=2){			// check if line nonempty
			tokenize(str,info);
			vector<string>::iterator it=info.begin();
			TrackInfo newtrack;
			// Give information to the newly creaeted track
			newtrack.set_number(number);
		
			if( it !=info.end() ){
				newtrack.set_artist(*it);
				it++;
			}		
			if( it !=info.end() ){
				newtrack.set_albumtitle(*it);
				it++;
			}
			if( it !=info.end() ){
				newtrack.set_title(*it);
				it++;
			}
			if( it !=info.end() ){
				newtrack.set_genre(*it);
				it++;
			}
			if( it !=info.end() ){
				newtrack.set_date(*it);
				it++;
			}
			if( it !=info.end() ){
				newtrack.set_time(atoi((*it).c_str()));
				it++;
			}
			if( it !=info.end() ){
				newtrack.set_comment(*it);
				it++;
			}
			content.addTrack(newtrack);
			info.clear(); // clear the container of linetokens
			number++;
		}
	}
}
/**
 * split the lines into usable tokens
 */
void CDInfoReader_csv::tokenize (const string& line, vector<string>& schnipsel) {
	// Skip first delimiter at beginning.
	string::size_type lastPos =  line.find_first_not_of(delimiter.c_str(), 0);
	// Find first "non-delimiter".
	string::size_type pos  = line.find_first_of(delimiter.c_str(), lastPos);
	while (string::npos != pos || string::npos != lastPos){
		// Found a token, add it to the vector.
		schnipsel.push_back(line.substr(lastPos, pos - lastPos));
		// Skip delimiters. 
		lastPos = line.find_first_not_of(delimiter.c_str(), pos);
		pos = line.find_first_of(delimiter, lastPos);
	}
}

/**
 * Constructors/Destructors
 */
extern "C" CDInfoReader* create(string infile){
	return new CDInfoReader_csv(infile);
	//return new CDInfoReader_csv;
}

extern "C" void destroy(CDInfoReader* todestroy){
	delete todestroy;
}

