/******************************************************************************
* Synopsis:
*-----------------------
* Dependencies:
*
*-----------------------
* CVS:
* $Author: ruschi $
* $Date: 2004/11/21 17:01:37 $
* $Revision: 1.3 $ 
*-----------------------------------------------------------------------------
*  Copyright (C) 2004 by Thomas Ruschival                               
*  ruschi@doomgarage.de                                                 
*                                                                       
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or    
*  (at your option) any later version.                                  
*                                                                       
*  This program is distributed in the hope that it will be useful,      
*  but WITHOUT ANY WARRANTY; without even the implied warranty of       
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
*  GNU General Public License for more details.                         
*                                                                       
*  You should have received a copy of the GNU General Public License    
*  along with this program; if not, write to the                        
*  Free Software Foundation, Inc.,                                      
*  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.            
*******************************************************************************/
#include "cdinforeader_m3u.h"

using namespace std;

CDInfoReader_m3u::CDInfoReader_m3u(string infile):CDInfoReader(infile){
	name = "m3u Parser";
	filetype = "m3u";
	description="Parser for standard m3u Playlists \n";
	description+="Version: $Revision: 1.3 $ Copyright (c) 2004 Thomas Ruschival \n";
}


CDInfoReader_m3u::CDInfoReader_m3u():CDInfoReader(){
	name = "m3u Parser";
	filetype = "m3u";
	description="Parser for standard m3u Playlists \n";
	description+="Version: $Revision: 1.3 $ Copyright (c) 2004 Thomas Ruschival \n";
}

/**
 * find out if the reader is suitable for the
 * source
 * @param infile string filename
 * @return fits int 0 if false, 1 if it fits
 */
int  CDInfoReader_m3u::matches(string infile){
	// find and extract the extension of the file
	int pos = infile.find_last_of('.',infile.size());
	string extension = infile.substr(pos+1,infile.size());
	// Make the fileextension lowercase
	std::transform(extension.begin(),extension.end(),extension.begin(),(int(*)(int))tolower);
	//Get information about file; determine by extension
	if(extension == filetype){
		return 1;
	}
	else{
		return 0;
	}
}


/**
 * parse the file, split the lines, generate trackinfos
 */
void CDInfoReader_m3u::readCDInfo (CDInfo &content) {
	int pos=0, oldpos=0;
	int number=1;
	char buffer[256];
	ifstream examplefile(this->source.c_str());
	if (! examplefile.is_open()){
		cout << "Error opening file"; exit (1);
	}
  
	while (! examplefile.eof() )  {
		examplefile.getline (buffer,255);
		const string str(buffer);
		if(str.substr(0,8) == "#EXTINF:"){
			cout << str.substr(0,8)<< "track \n";		// check if line is Trackinformation
			TrackInfo newtrack;
			// Give information to the newly created track
			newtrack.set_number(number);
			// the time is betweern the #EXTINF: and the comma
			oldpos = str.find_first_of(":",0);
			pos = str.find_first_of(",",oldpos);
			newtrack.set_time(atoi(str.substr(oldpos+1,pos-oldpos-1).c_str()));
			// Artist and songname are separed by a dash (-)
			oldpos = pos+1;
			pos = str.find_first_of("-",oldpos);
			newtrack.set_artist(str.substr(oldpos,pos-oldpos));
			// Title is the rest of the line
			oldpos = pos+1;
			newtrack.set_title(str.substr(oldpos+1,str.size()-oldpos+1));
			content.addTrack(newtrack);
			number++;
		}
	}
}


/**
 * Constructors/Destructors
 */
extern "C" CDInfoReader* create(string infile){
	return new CDInfoReader_m3u(infile);
}

extern "C" void destroy(CDInfoReader* todestroy){
	delete todestroy;
}
