#include "clientsocket.h"


/**
 * Constructors/Destructors
 */
ClientSocket::ClientSocket(const char* host , int port):Socket(){
	
	connect(host,port);
}
  
/**
 * Methods
 */

/**
 * connect to a server by using its hostname or ip-address and a port
 * @param hostname either ip-address in dotted digit format or FQ hostname
 * @return int 1 if sucessful, 0 otherwise
*/
int ClientSocket::connect (const char* hostname, int port) {
	// Setup and init the sock_addr_in struct, needed by real connect function
	struct sockaddr_in SrvrAdrs;
	memset( &SrvrAdrs, 0, sizeof( SrvrAdrs ) );
	// If szHostAddr looks like '127.0.0.1' then it's easy, just
	// call inet_addr to convert the string into a netword address ID integer
	if( isdigit(hostname[0]) )
		SrvrAdrs.sin_addr.s_addr = inet_addr( hostname );
	else{
			// Otherwise, it may be a host name. Get it's IP address and use that.
			// For example, szHostAddr may be www.acme.com
		struct hostent * pHostEnt = gethostbyname( hostname );
		if( pHostEnt == NULL )
		{
			throw SocketException("Could not connect","Unable to determine IP for hostname", errno );
			return( -1 );
		}
		SrvrAdrs.sin_addr.s_addr = *(long*)pHostEnt->h_addr_list[0];
	}
	SrvrAdrs.sin_family = AF_INET;
	// Call htons, to convert our local pc short to format compatible with the 'net'
	SrvrAdrs.sin_port = htons( port );
	// finally, call the other version of MSocket::Connect, with the struct we set up
	return( connect((struct sockaddr*) &SrvrAdrs , (int)sizeof( SrvrAdrs ) ) );
}
/**
 * Connect to a server if we got a sockadd struct and the lenght
 * return 1 if sucessful, 0 otherwise 
 */
int ClientSocket::connect(sockaddr* addr, int length) {
	if( sock < 0 ){
		throw SocketException("Could not connect.","socket not initialized",errno);
		return(0);
	}
	// Call the connect system function to actually do connect.
	int test = ::connect( sock, (struct sockaddr*)addr, length);
	if( test < 0){ // If there was an error, and we should throw it.
		throw SocketException("Connect failed", "no idea why, check errno!",errno);
		return 0;
	}
	return(1);
}

/**
 *  Wrapper for output
 */
ClientSocket & operator<<(ClientSocket & out, const char *cpStr){
	out.writeLine(cpStr);
	return (out);
}


/**
 *  Wrapper for input
 */
ClientSocket & operator>>(ClientSocket & in, char *cpStr){
	in.readLine(cpStr, 0x0FFFF);	// No good number here, really
	return (in);
}
